@@ -83,6 +83,7 @@ class Metabase_API():
     else:
       raise ValueError('There is no {} with the id {}'.format(item_type, item_id))
   
+
   
   def get_item_id(self, item_type, item_name, collection_id=None, collection_name=None):
     
@@ -90,7 +91,7 @@ class Metabase_API():
 
     if not collection_id:
       if not collection_name:
-        # Collection name/id is not privided. Searching in all collections 
+        # Collection name/id is not provided. Searching in all collections 
         item_IDs = [ i['id'] for i in self.get("/api/{}/".format(item_type)) if i['name'] == item_name 
                                                                             and i['archived'] == False ]
       else:
@@ -118,6 +119,7 @@ class Metabase_API():
     
     return item_IDs[0] 
   
+
   
   def get_collection_id(self, collection_name):
     collection_IDs = [ i['id'] for i in self.get("/api/collection/") if i['name'] == collection_name ]
@@ -129,6 +131,7 @@ class Metabase_API():
     
     return collection_IDs[0] 
 
+
   
   def get_segment_id(self, segment_name, table_id=None):
     segment_IDs = [ i['id'] for i in self.get("/api/segment/") if i['name'] == segment_name 
@@ -141,6 +144,7 @@ class Metabase_API():
     return segment_IDs[0]
 
 
+
   def get_db_id(self, db_name):
     db_IDs = [ i['id'] for i in self.get("/api/database/") if i['name'] == db_name ]
     
@@ -151,6 +155,7 @@ class Metabase_API():
     
     return db_IDs[0]
   
+
   
   def get_table_id(self, table_name, db_name=None, db_id=None):
     tables = self.get("/api/table/")
@@ -170,6 +175,7 @@ class Metabase_API():
     return table_IDs[0]
 
 
+
   def get_db_id_from_table_id(self, table_id):
     tables = [ i['db_id'] for i in self.get("/api/table/") if i['id'] == table_id ]
     
@@ -178,29 +184,46 @@ class Metabase_API():
     
     return tables[0]
 
+
   
   def get_columns_name_id(self, table_name=None, db_name=None, table_id=None, db_id=None, verbose=False):
-    '''Return a dictionary with col_id key and col_name value, 
-       for the given table_id/table_name in the given db_id/db_name.
-       We need to have the table_name because the endpoint /api/database/:db_id/fields shows only table name
+    '''
+    Return a dictionary with col_id key and col_name value, 
+    for the given table_id/table_name in the given db_id/db_name.
+    We need to have the table display name because the endpoint /api/database/:db_id/fields 
+    shows only the table display name (not its actual name in the DB, and also not its ID).
     '''
     if not table_name:
       if not table_id:
         raise ValueError('Either the name or id of the table must be provided.')
       table_name = self.get_item_name(item_type='table', item_id=table_id)
 
+    # getting db_id
     if not db_id:
-      if not db_name:
+      if db_name:
+        db_id = self.get_db_id(db_name)
+      else:
         if not table_id:
           table_id = self.get_table_id(table_name)
         db_id = self.get_db_id_from_table_id(table_id)
-      db_id = self.get_db_id(db_name)
-    
-    # Getting column names and id's 
+        
+    # Getting column names and IDs 
     return {i['name']: i['id'] for i in self.get("/api/database/{}/fields".format(db_id)) 
                                if i['table_name'] == table_name}
-  
-  
+
+
+
+  def friendly_names_is_disabled(self):
+    '''
+    The endpoint /api/database/:db-id/fields which is used in the function get_column_name_id relies on the display name of fields. 
+    If "Friendly Table and Field Names" (in Admin Panel > Settings > General) is not disabled, it changes the display name of fields.
+    So it is important to make sure this setting is disabled, before running the get_column_name_id function.
+    '''
+    friendly_name_setting = [ i['value'] for i in self.get('/api/setting') if i['key'] == 'humanization-strategy' ][0]
+    return friendly_name_setting == 'none'  # 'none' means disabled
+
+
+
   @staticmethod
   def verbose_print(verbose, msg):
     if verbose:
@@ -215,8 +238,9 @@ class Metabase_API():
   def create_card(self, card_name=None, collection_name=None, collection_id=None, 
                   db_name=None, db_id=None, table_name=None,  table_id=None, 
                   column_order='db_table_order', custom_json=None, verbose=False, return_card=False):
-    """Create a card using the given arguments utilizing the endpoint 'POST /api/card/'. 
-       If collection is not given, the root collection is used.
+    """
+    Create a card using the given arguments utilizing the endpoint 'POST /api/card/'. 
+    If collection is not given, the root collection is used.
     
     Keyword arguments:
     card_name -- the name used to create the card (default None) 
@@ -237,6 +261,7 @@ class Metabase_API():
     return_card --  whather to return the created card info (default False)
     """
     if custom_json:
+      # checking whether the provided json is complete or not
       complete_json = True
       for item in ['name', 'dataset_query', 'display']:
         if item not in custom_json:
@@ -244,6 +269,10 @@ class Metabase_API():
           self.verbose_print(verbose, 'The provided json is detected as partial.')
           break
       
+      # Fixing the issue #10
+      if custom_json.get('description') == '': 
+        custom_json['description'] = None
+
       if complete_json:
         # Adding visualization_settings if it is not present in the custom_json
         if 'visualization_settings' not in custom_json:
@@ -258,7 +287,9 @@ class Metabase_API():
           print('Card Creation Failed.\n', res)
           return res
     
-    # making sure we have table_id and table_name and db_id
+    # making sure we have the required data
+    if not card_name and (not custom_json or not custom_json.get('name')):
+      raise ValueError("A name must be provided for the card (either as card_name argument or as part of the custom_json ('name' key)).")
     if not table_id:
       if not table_name:
         raise ValueError('Either the name or id of the table must be provided.')
@@ -268,7 +299,7 @@ class Metabase_API():
     if not db_id:
       db_id = self.get_db_id_from_table_id(table_id)
 
-    # getting collection_id if a collection is specified
+    # getting collection_id if it is not given
     if not collection_id:
       if not collection_name:
         self.verbose_print(verbose, 'No collection name or id is provided. Will create the card at the root ...')
@@ -276,6 +307,10 @@ class Metabase_API():
         collection_id = self.get_collection_id(collection_name)
     
     if type(column_order) == list:
+
+      if not self.friendly_names_is_disabled():
+        raise ValueError('Please disable "Friendly Table and Field Names" from Admin Panel > Settings > General, and try again.')
+      
       column_name_id_dict = self.get_columns_name_id(db_id=db_id, 
                                                      table_id=table_id, 
                                                      table_name=table_name, 
@@ -289,6 +324,10 @@ class Metabase_API():
       column_id_list_str = [['field-id', i] for i in column_id_list]
 
     elif column_order == 'db_table_order':  # default
+      
+      if not self.friendly_names_is_disabled():
+        raise ValueError('Please disable "Friendly Table and Field Names" from Admin Panel > Settings > General, and try again.')
+
       ### Finding the actual order of columns in the table as they appear in the database
       # creating a temporary card for retrieving column ordering
       json_str = """{{'dataset_query': {{'database': {1},
@@ -303,17 +342,17 @@ class Metabase_API():
         print('Card Creation Failed!')
         return res
       ordered_columns = [ i['name'] for i in res['result_metadata'] ]  # retrieving the column ordering
-    
+
       # deleting the temporary card
       card_id = res['id']
       self.delete("/api/card/{}".format(card_id))  
-      
+
       column_name_id_dict = self.get_columns_name_id(db_id=db_id, 
                                                      table_id=table_id, 
                                                      table_name=table_name, 
                                                      verbose=verbose)
-      column_id_list = [column_name_id_dict[i] for i in ordered_columns]
-      column_id_list_str = [['field-id', i] for i in column_id_list]
+      column_id_list = [ column_name_id_dict[i] for i in ordered_columns ]
+      column_id_list_str = [ ['field-id', i] for i in column_id_list ]
     
     elif column_order == 'alphabetical':
       column_id_list_str = None
@@ -359,10 +398,12 @@ class Metabase_API():
       print('Card Creation Failed.\n', res)
       return res
   
+
   
   def create_segment(self, segment_name, column_name, column_values, segment_description='', 
                      db_name=None, db_id=None, table_name=None, table_id=None, return_segment=False):
-    """Create a segment using the given arguments utilizing the endpoint 'POST /api/segment/'. 
+    """
+    Create a segment using the given arguments utilizing the endpoint 'POST /api/segment/'. 
     
     Keyword arguments:
     segment_name -- the name used for the created segment.
@@ -383,7 +424,10 @@ class Metabase_API():
     if not table_name:
       table_name = self.get_item_name(item_type='table', item_id=table_id)
     db_id = self.get_db_id_from_table_id(table_id)
-      
+    
+    if not self.friendly_names_is_disabled():
+      raise ValueError('Please disable "Friendly Table and Field Names" from Admin Panel > Settings > General, and try again.')
+
     colmuns_name_id_mapping = self.get_columns_name_id(table_name=table_name, db_id=db_id)
     column_id = colmuns_name_id_mapping[column_name]
     
@@ -402,12 +446,14 @@ class Metabase_API():
       return res
 
 
+
   def copy_card(self, source_card_name=None, source_card_id=None, 
                 source_collection_name=None, source_collection_id=None,
                 destination_card_name=None, 
                 destination_collection_name=None, destination_collection_id=None,
                 postfix='', verbose=False):
-    """Copy the card with the given name/id to the given destination collection. 
+    """
+    Copy the card with the given name/id to the given destination collection. 
     
     Keyword arguments:
     source_card_name -- name of the card to copy (default None) 
@@ -450,6 +496,10 @@ class Metabase_API():
     card_json['collection_id'] = destination_collection_id
     card_json['name'] = destination_card_name
     
+    # Fixing the issue #10
+    if card_json.get('description') == '': 
+      card_json['description'] = None
+
     # saving as a new card
     res = self.create_card(custom_json=card_json, verbose=verbose, return_card=True)
     
@@ -457,11 +507,13 @@ class Metabase_API():
     return res['id']
 
 
+
   def copy_pulse(self, source_pulse_name=None, source_pulse_id=None, 
                  source_collection_name=None, source_collection_id=None,
                  destination_pulse_name=None, 
                  destination_collection_id=None, destination_collection_name=None, postfix=''):
-    """Copy the pulse with the given name/id to the given destination collection. 
+    """
+    Copy the pulse with the given name/id to the given destination collection. 
     
     Keyword arguments:
     source_pulse_name -- name of the pulse to copy (default None) 
@@ -507,12 +559,14 @@ class Metabase_API():
     self.post('/api/pulse', json=pulse_json)
     
   
+
   def copy_dashboard(self, source_dashboard_name=None, source_dashboard_id=None, 
                      source_collection_name=None, source_collection_id=None,
                      destination_dashboard_name=None, 
                      destination_collection_name=None, destination_collection_id=None,
                      deepcopy=False, postfix=''):
-    """Copy the dashboard with the given name/id to the given destination collection. 
+    """
+    Copy the dashboard with the given name/id to the given destination collection. 
     
     Keyword arguments:
     source_dashboard_name -- name of the dashboard to copy (default None) 
@@ -594,13 +648,17 @@ class Metabase_API():
         self.delete('/api/dashboard/{}/cards'.format(dup_dashboard_id), params={'dashcardId':dash_card_id})
         # adding the new card to the duplicated dashboard
         self.post('/api/dashboard/{}/cards'.format(dup_dashboard_id), json=card_json)
+      
+    return dup_dashboard_id
   
   
+
   def copy_collection(self, source_collection_name=None, source_collection_id=None, 
                       destination_collection_name=None,
                       destination_parent_collection_name=None, destination_parent_collection_id=None, 
                       deepcopy_dashboards=False, postfix='', child_items_postfix='', verbose=False):
-    """Copy the collection with the given name/id into the given destination parent collection. 
+    """
+    Copy the collection with the given name/id into the given destination parent collection. 
     
     Keyword arguments:
     source_collection_name -- name of the collection to copy (default None) 
@@ -693,23 +751,12 @@ class Metabase_API():
         self.copy_pulse(source_pulse_id=pulse_id,
                         destination_collection_id=destination_parent_collection_id,
                         destination_pulse_name=destination_pulse_name)
-    
-    
-  @staticmethod
-  def make_json(raw_json, prettyprint=False):
-    """Turn the string copied from the Inspect->Network window into a Dict."""
-    json = eval(raw_json.replace('null', 'None') \
-                        .replace('false', 'False') \
-                        .replace('true', 'True')
-               )
-    if prettyprint:
-      import pprint
-      pprint.pprint(json)
-    return json
   
   
+
   def move_to_archive(self, item_type, item_name=None, item_id=None, 
                       collection_name=None, collection_id=None, table_id=None, verbose=False):
+    '''Archive the given item. For deleting the item use the 'delete_item' function.'''
     assert item_type in ['card', 'dashboard', 'collection', 'pulse', 'segment']
     if not item_id:
       if not item_name:
@@ -733,3 +780,54 @@ class Metabase_API():
       print('Archiving Failed.')
     
     return res
+
+      
+
+  def delete_item(self, item_type, item_name=None, item_id=None, 
+                  collection_name=None, collection_id=None, table_id=None, verbose=False):
+    '''
+    Delete the given item. Use carefully (this is different from archiving).
+    Currently Collections and Segments cannot be deleted using the Metabase API.
+    '''
+    assert item_type in ['card', 'dashboard', 'pulse']
+    if not item_id:
+      if not item_name:
+        raise ValueError('Either the name or id of the {} must be provided.'.format(item_type))
+      item_id = self.get_item_id(item_type, item_name, collection_id, collection_name)
+      
+    return self.delete('/api/{}/{}'.format(item_type, item_id))
+
+
+
+  def update_col_type_in_data_model(self, column_type='Category', column_id=None, column_name=None, 
+                                    table_name=None, table_id=None, db_name=None, db_id=None):
+    '''Update the column type of the specified column to the provided "column_type", in the data model.'''
+    assert column_type in ['Category']  # will implement other types later
+    
+    # making sure we have the data we need
+    if not column_id and not column_name:
+      raise ValueError('Either the name or id of the column needs to be provided.')
+    if not table_id and not table_name:
+      raise ValueError('Either the name or id of the table needs to be provided.')
+      
+    if not column_id:
+      columns_name_id_mapping = self.get_columns_name_id(table_name=table_name, table_id=table_id)
+      column_id = columns_name_id_mapping.get(column_name)
+      if column_id is None:
+        raise ValueError('There is no column named {} in the provided table'.format(column_name))
+        
+    return self.put('/api/field/{}'.format(column_id), json={'special_type':'type/{}'.format(column_type)})
+      
+
+
+  @staticmethod
+  def make_json(raw_json, prettyprint=False):
+    """Turn the string copied from the Inspect->Network window into a Dict."""
+    json = eval(raw_json.replace('null', 'None') \
+                        .replace('false', 'False') \
+                        .replace('true', 'True')
+               )
+    if prettyprint:
+      import pprint
+      pprint.pprint(json)
+    return json
